(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit AutoRunApp;

interface

uses
  Windows, Classes, Forms, Registry;

type
  TRegLocation   = (rlCurrentUser, rlLocalMachine);
  TAutoRunWhere  = (awRun, awRunOnce, awRunServices, awRunServicesOnce);
  TAutoAppRunner = class(TComponent)
  private
    { Dclarations prives }
    FRegLocation  : TRegLocation;
    FAutoRunWhere : TAutoRunWhere;
    FAutoRun      : Boolean;
    FParam,
    FAppName      : String;
    FOtherAppName: String;

    procedure ModifyRegistry(Location:TRegLocation; AppName, Param: String; Auto: boolean);
    procedure SetRegLocation(Value: TRegLocation);
    procedure SetAutoRunWhere(Value: TAutoRunWhere);
    procedure SetAutoRun(Value: Boolean);
    procedure SetAppName(Value: String);
    procedure SetParam(Value: String);
    function  RunTime: Boolean;
    procedure SetOtherAppName(const Value: String);
  protected
    { Dclarations protges }
    RunKey : String;
    Procedure VerifRunWhere;
  public
    { Dclarations publiques }
    constructor Create(aOwner: TComponent); override;
  published
    { Dclarations publies }
    property RegLocation : TRegLocation  read FRegLocation  write SetRegLocation;
    property AutoRunWhere: TAutoRunWhere read FAutoRunWhere write SetAutoRunWhere;   {*** AddOn ***}
    property AutoRun     : Boolean       read FAutoRun      write SetAutoRun;
    property AppName     : String        read FAppName      write SetAppName;
    property Param       : String        read FParam        write SetParam;
    property OtherAppName: String        read FOtherAppName write SetOtherAppName;
  end;

implementation

resourcestring
  RunWhere = '\Software\Microsoft\Windows\CurrentVersion\';

constructor TAutoAppRunner.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);

  if(not RunTime) then
    AppName := 'MyApp';
  Runkey := RunWhere + 'Run';
  FOtherAppName := '';
end;


procedure TAutoAppRunner.ModifyRegistry(Location:TRegLocation; AppName, Param: String; Auto: boolean);
var Reg: TRegistry;
begin
  if(RunTime) then begin
    Reg := TRegistry.Create;
    try
      // Supprime toute les clefs
      Reg.RootKey := HKEY_CURRENT_USER;

      if(Reg.OpenKey(RunKey, False)) then
      try
        Reg.DeleteValue(FAppName);
      finally
        Reg.CloseKey;
      end;

      Reg.RootKey := HKEY_LOCAL_MACHINE;

      if(Reg.OpenKey(RunKey, False)) then
      try
       Reg.DeleteValue(FAppName);
      finally
        Reg.CloseKey;
      end;

      // Puis recris
      if(Auto) then begin
        if Location = rlCurrentUser then
          Reg.RootKey := HKEY_CURRENT_USER
        else
          Reg.RootKey := HKEY_LOCAL_MACHINE;

        if(Reg.OpenKey(RunKey, False)) then
        try
          if(OtherAppName = '') then
            Reg.WriteString(AppName, Application.ExeName + ' ' + Param)
          else
            Reg.WriteString(AppName, OtherAppName + ' ' + Param)
        finally
          Reg.CloseKey;
        end;
      end;
    finally
      Reg.Free;
    end;
  end;
end;

Procedure TAutoAppRunner.VerifRunWhere;
begin
  If(FRegLocation = rlCurrentUser) then
    If((FAutoRunWhere = awRunServices) or (FAutoRunWhere = awRunServicesOnce)) then
      FAutoRunWhere := awRun;
end;

procedure TAutoAppRunner.SetRegLocation(Value: TRegLocation);
begin
  if(FRegLocation <> Value) then begin
    ModifyRegistry(Value, FAppName, FParam, FAutoRun);
    FRegLocation := Value;
    VerifRunWhere;
  end;
end;

procedure TAutoAppRunner.SetAutoRunWhere(Value: TAutoRunWhere);
begin
  if(FAutoRunWhere <> Value) then begin
    FAutoRunWhere := Value;
    VerifRunWhere;
    case(FAutoRunWhere) of
      awRun             : RunKey := 'Run';
      awRunOnce         : RunKey := 'RunOnce';
      awRunServices     : RunKey := 'RunServices';
      awRunServicesOnce : RunKey := 'RunServicesOnce';
    end;
    Runkey := RunWhere + Runkey;
  end;
end;

procedure TAutoAppRunner.SetAutoRun(Value: Boolean);
begin
  if((FAutoRun <> Value) and RunTime) then begin
    FAutoRun := Value;
    VerifRunWhere;
    ModifyRegistry(FRegLocation, FAppName, FParam, FAutoRun);
  end;
end;

procedure TAutoAppRunner.SetAppName(Value: String);
begin
  if(csLoading in ComponentState) then begin
    ModifyRegistry(FRegLocation, FAppName, FParam, false);
    FAppName := Value;
    ModifyRegistry(FRegLocation, FAppName, FParam, FAutoRun);
  end else
    if(not RunTime) then
      if (Value <> '') and (FAppName <> Value) then
        FAppName := Value
end;

procedure TAutoAppRunner.SetParam(Value: String);
begin
  if(csLoading in ComponentState) then begin
    ModifyRegistry(FRegLocation, FAppName, FParam, false);
    FParam := Value;
    ModifyRegistry(FRegLocation, FAppName, FParam, FAutoRun);
  end else
    if(not RunTime) then
      if((Value <> '') and (FParam <> Value)) then
        FParam := Value
end;

function TAutoAppRunner.RunTime: Boolean;
begin
  Result := not ((csDesigning in ComponentState) or
                 (csReading   in ComponentState) or
                 (csLoading   in ComponentState));
end;

procedure TAutoAppRunner.SetOtherAppName(const Value: String);
begin
  FOtherAppName := Value;
end;

end.
