(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit FileTools;

interface

{$IFNDEF Win32}
  type Int64 = Integer;
{$ENDIF}

USES Graphics, Consts, SysUtils, Classes, WinProcs;

Type
  tPercentProc   = procedure (Percent : integer; St : String; var abort : boolean);
  TDriveType     = (Unknown, no_root, Removable, Fixed, Remote, CDROM, RamDisk);
   { Inconue, pas de rperoire racine, lecteur amovible (disquette), lecteur fixe (disque dur),
     Lecteur rseau, Lecteur CD-ROM, Ram disk }

(* Routines sur les dossiers *)

function NormalDir (Path : string)                                   : string;
function UnixPathToDosPath(Path: String)                             : string;
function DosPathToUnixPath(Path: String)                             : string;
function DirExists (const Name: string)                              : Boolean;
function Replacefolder(Dossier: string)                              : string;
procedure MakeAllDirs(Dossier: string);

(* Routines pour la gestion des tailles sur les fichiers/dossiers *)

function SizeToStr (Size : Int64)                                    : string; overload;
function SizeToStr (Size : Int64; const units: array of string)      : string; overload;
function SizeOfDir (Path : string)                                   : int64; platform;
function SizeOfFile(FileName : string)                               : int64;

(* Routines de gestions de Date sur les fichiers *)

function TimToStr (Time : int64)                                     : string;
function FileDate(FileName: String)                                  : TDateTime;
procedure SetFileDate(FileName: String;d: TDateTime);

(* Gestion des lettres de lecteur et sur les lecteurs *)

function GetNumDrive                                                 : integer;
function LetterToNumb (Letter : char)                                : Byte;
function DriveType (DriveLetter : char)                              : tDriveType;
function DriveTypeStr (DriveLetter : char)                           : String;
function DiskInDrive (Drive : char)                                  : boolean;

function EnougthSpace (DriveUnit : char; Size : Int64)               : boolean;

function IsCDROM (DriveNum : Smallint)                               : boolean;
function ChercheCDROM                                                : String;

(* Routines sur les attributs et les types des fichiers *)

function FileAttribue(SRec : TSearchRec)                             : string; overload; platform;
function FileAttribue(FileName : TFileName)                          : string; overload; platform;
function FileExtType (FileName : TFileName)                          : string;
function FileIcon    (FileName : TFileName)                          : TIcon; overload;
function FileIcon    (FileName : TFileName; getSmallIcon,
                                            getOpenIcon: Boolean)    : HIcon; overload;

(* Gnration d'un nom pour un fichier temporaire *)

function GenerateFileName(const Dest: string = '';
                          const Base: string = 'TMP')                : string;

(* Routines agissant sur les fichiers *)

function  FileIsUse(FileName: TFileName)                             : boolean;
procedure FileCopy(const SrcName, DestName: tFileName; Percent : tPercentProc);

implementation

uses WinTypes, Registry, WinInfos, tSysUtils, PaintUtils, tShellAPI, ShellAPI;

resourcestring
  cRegDefaultIcon = '\DefaultIcon';
  SFOpenError     = 'Ne peut ouvrir le fichier.';
  SFCreateError   = 'Ne peut creer le fichier.';

const GrandeurOctee : ARRAY [0..6] OF String = ('octet(s)', 'Ko', 'Mo', 'Go', 'To', 'Po', 'Eo');

function Replacefolder(Dossier: string): string;
var
  Chaine: String;
  index : integer;
begin
  result := Dossier;
  if (pos('<', result) = 1) and (pos('>', result) > 1) then
  begin
    index := pos('>', result);
    if pos(upperCase('<commondir>' ), upperCase(result)) <> 0 then Chaine := NormalDir(GetCommonDir)  else
    if pos(upperCase('<tempdir>'   ), upperCase(result)) <> 0 then Chaine := NormalDir(GetTempDir)    else
    if pos(upperCase('<windowsdir>'), upperCase(result)) <> 0 then Chaine := NormalDir(GetWindowsDir) else
    if pos(upperCase('<systemdir>' ), upperCase(result)) <> 0 then Chaine := NormalDir(GetSystemDir)  else
    raise Exception.Create(Dossier + ' incorrect.');
    Delete(Chaine, length(chaine), 1);
    Delete(Result, 1, index);
    result := chaine + result;
  end;
end;

procedure MakeAllDirs(Dossier: string);
begin
  if(Dossier = EmptyStr) then Exit;
  if(IsDelimiter('\', Dossier, length(Dossier))) then
    MakeAllDirs(Copy(Dossier, 1, LastDelimiter('\', Dossier) - 1));
  if(not DirExists(Dossier)) then begin
    MakeAllDirs(Copy(Dossier, 1, LastDelimiter('\', Dossier) - 1));
    MkDir(Dossier);
  end;
end;

function TimToStr (Time : int64) : string;
var sec, min, heure : integer;
begin
  heure := (((Time div 1000) div 60) div 60);
  min   := ((Time div 1000) div 60) - (heure * 60);
  sec   := (Time div 1000) - (heure * 3600) - (min * 60);
  result := LockSizeText(IntToStr(heure), 2, '0') + ':' +
            LockSizeText(IntToStr(min  ), 2, '0') + ':' +
            LockSizeText(IntToStr(sec  ), 2, '0');
end;

function SizeOfDir (Path : string) : int64;
var SRec   : TSearchRec;
    RetVal : integer;
    OldLen : Integer;
begin
  if not IsPathDelimiter (path, length (path))
  then path := path + '\';
  result := 0;
  OldLen := Length (Path);
  RetVal := FindFirst (Path + '*.*', faAnyFile, SRec);
  While RetVal = 0 do
  begin
    If (SRec.Attr and (faDirectory or faVolumeID)) = 0 then
    begin
      Inc (Result, SRec.Size);
    end;
    RetVal := FindNext (SRec);
  end;
  SysUtils.FindClose (SRec);
  RetVal := FindFirst (Path + '*.*', faDirectory, SRec);
  While RetVal = 0 Do
  begin
    if (SRec.Attr and faDirectory) <> 0 then
    begin
      if (SRec.Name <> '.') and (SRec.Name <> '..') then
      begin
        path := path + SRec.Name + '\';
        Inc (Result, SizeOfDir (Path));
        Delete (path, oldlen + 1, length (path));
      end;
    end;
    RetVal := FindNext (SRec);
  end;
  SysUtils.FindClose (SRec);
end;

function SizeOfFile(FileName : string): int64;
var F: file of Byte;
begin
  AssignFile(F, FileName);
  FileMode := 0;
  {$I-}
  Reset(F);
  {$I+}
  if IOResult = 0 then
  begin
    result := FileSize(F);
    CloseFile(F);
  end else
    result  := 0;
end;

function FileDate(FileName: String): TDateTime;
var Handle: Integer;
begin
  result := Now;
  Handle := FileOpen(FileName, fmOpenRead or fmShareDenyNone);
  if Handle <> -1 then
  try
    result := FileDateToDateTime(FileGetDate(Handle));
  finally
    FileClose(Handle);
  end;
end;

procedure SetFileDate(FileName: String;d: TDateTime);
var Handle: Integer;
begin
  Handle := FileOpen(FileName, fmOpenWrite);
  if Handle <> -1 then
  try
    FileSetDate(Handle, DateTimeToFileDate(d));
  finally
    FileClose(Handle);
  end;
end;

function GetNumDrive : integer;
var i :integer;
begin
  result := 0;
  for I := Ord ('A') to ord ('Z') do
  begin
    if DriveType (Chr (I)) <> no_root
      then result := I - Ord ('A') + 1;
  end;
end;

function DiskInDrive (Drive : char) : boolean;
var ErrorMode : word;
begin
  Drive := UpCase (Drive);
  If not (Drive in ['A'..'Z'])
    then raise EConvertError.Create ('Lettre de lecteur invalide');
  ErrorMode := SetErrorMode (SEM_FailCriticalErrors);
  try
    Result := not (DiskSize (Ord (Drive) - $40) = -1)
  finally
    SetErrorMode (ErrorMode);
  end;
end;

function IsCDROM (DriveNum : Smallint) : boolean; assembler;
asm
  MOV AX, 1500h { Cherche MSCDEX }
  XOR BX, BX
  INT 2fh
  OR  BX,BX
  JZ  @Finish
  MOV AX, 150Bh { vrifie que le driver de CD est utilis }
  MOV CX, DriveNum
  INT 2fh
  OR  AX, AX
  @Finish:
end;

function ChercheCDROM : String;
var DriveNum : integer;
begin
  Result := '';
  For DriveNum := 0 to 25
    do if IsCDROM (DriveNum)
         then Result := Result + UpCase (Chr (DriveNum + ord ('a')));
end;

function SizeToStr (Size : Int64; const units: array of string)      : string; overload;
var Sz       : int64;
    Num, Max : integer;
begin
  Sz  := Size;
  Max := High (units);
  Num := 0;
  While Sz >= 1000
  DO BEGIN
       Sz := Sz DIV 1024;
       Inc (Num);
     END;
  IF num > Max THEN Num := Max;
  result := RealToStr (Size / (exp (Num * Ln (1024))), 2) + ' ' + units[Num];
end;

function SizeToStr (Size : Int64) : String;
begin
  result := SizeToStr(Size, GrandeurOctee);
end;

function DriveTypeStr (DriveLetter : char) : String;
var Drv : TDriveType;
begin
  Drv := DriveType (DriveLetter);
  Case Drv of
    Unknown   : Result := 'Le type du lecteur ne peut tre dtermin.';
    no_root   : Result := 'Le rpertoire racine n''existe pas.';
    Removable : Result := 'Lecteur amovible.';
    Fixed     : Result := 'Lecteur fixe.';
    Remote    : Result := 'Lecteur rseau.';
    CDROM     : Result := 'Lecteur de CD-ROM.';
    RamDisk   : Result := 'RAM Disk.';
  end;
end;

function DriveType (DriveLetter : char) : tDriveType;
{$IFNDEF Win32}
var i :word;
begin
  DriveLetter := UpCase (DriveLetter);
  i := GetDriveType (ord (DriveLetter) - ord ('A'));
  case i of
{$ELSE}
begin
  case GetDriveType (PChar (DriveLetter + ':\')) of
    0               : result := Unknown;
    1               : result := no_root;
    DRIVE_CDROM     : result := CDROM;
    DRIVE_RAMDISK   : result := RamDisk;
{$ENDIF}
    DRIVE_REMOVABLE : result := Removable;
    DRIVE_FIXED     : result := Fixed;
    DRIVE_REMOTE    : result := Remote;
    else              result := Unknown;
  end;
end;

function EnougthSpace (DriveUnit : char;Size : Int64) : boolean;
var Dsk : integer;
begin
  Dsk := ord (upcase (DriveUnit)) - 64;
  result := DiskFree (Dsk) > Size;
end;

procedure FileCopy(const SrcName, DestName: tFileName; Percent : tPercentProc);
var Annuler : boolean;
var
  CopyBuffer: Pointer;
  TimeStamp, BytesCopied: Longint;
  Source, Dest: Integer;
  TotalBytes  : longint;
  TotalProce  : longint;
const
  ChunkSize: Longint = 8192;
begin
  TimeStamp := FileAge(SrcName);
  GetMem(CopyBuffer, ChunkSize);
  try
    Source := FileOpen(SrcName, fmShareDenyWrite);
    if Source < 0
   {$IFNDEF Win32}
     then raise EFOpenError.CreateFmt (LoadStr (SFOpenError), [SrcName]);
   {$ELSE}
     then raise EFOpenError.CreateFmt (SFOpenError, [SrcName]);
   {$ENDIF}
    try
      Dest := FileCreate(DestName);
      if Dest < 0 
     {$IFNDEF Win32}
       then raise EFCreateError.CreateFmt (LoadStr (SFCreateError), [DestName]);
     {$ELSE}
       then raise EFCreateError.CreateFmt (SFCreateError, [DestName]);
     {$ENDIF}
      try
        TotalProce := 0;
        TotalBytes := FileSeek (Source,0,2);
        FileSeek (Source, 0, 0);
        BytesCopied := 0;
        repeat
          TotalProce := TotalProce + BytesCopied;
          percent ((TotalProce * 100) DIV TotalBytes, ExtractFileName (DestName), Annuler);
          IF Annuler Then Break;
          BytesCopied := FileRead(Source, CopyBuffer^, ChunkSize);
          if BytesCopied > 0 then
            FileWrite(Dest, CopyBuffer^, BytesCopied);
        until BytesCopied < ChunkSize;
      finally
        FileSetDate (Dest, TimeStamp);
        FileClose (Dest);
      end;
    finally
      FileClose(Source);
    end;
  finally
    FreeMem(CopyBuffer, ChunkSize);
    IF Annuler then SysUtils.DeleteFile (DestName);
  end;
end;

function FileAttribue (SRec : TSearchRec) : string;
var Archive, ReadOnly, Hidden, System : boolean;
begin
  Archive  := SRec.Attr and faArchive > 0;
  ReadOnly := SRec.Attr and faReadOnly > 0;
  Hidden   := SRec.Attr and faHidden > 0;
  System   := SRec.Attr and faSysFile > 0;
  result   := '';
  If Archive  then result := result + 'A';
  If Hidden   then result := result + 'H';
  If ReadOnly then result := result + 'R';
  If System   then result := result + 'S';
end;

function FileAttribue(FileName : TFileName) : string;
var Archive, ReadOnly, Hidden, System : boolean;
    Attr                              : Integer;
begin
  Attr := FileGetAttr(FileName);
  Archive  := Attr and faArchive > 0;
  ReadOnly := Attr and faReadOnly > 0;
  Hidden   := Attr and faHidden > 0;
  System   := Attr and faSysFile > 0;
  result := '';
  If Archive  then result := result + 'A';
  If Hidden   then result := result + 'H';
  If ReadOnly then result := result + 'R';
  If System   then result := result + 'S';
end;

function LetterToNumb (Letter : char) : Byte;
begin
  Result := ord (upcase (Letter)) - 64;
end;

function FileExtType (FileName : TFileName) : string;
var s, Ext: string;
    Reg   : TRegistry;
begin
  Ext := ExtractFileExt(FileName); Ext := Copy(Ext, 2, length(Ext) - 1);
  s := '';
  Reg  := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    if Reg.OpenKey ('.' + Ext, false) then
    begin
      s := Reg.ReadString ('');
      Reg.CloseKey;
      if s <> '' then
      begin
        if Reg.OpenKey (s, false)
          then s := Reg.ReadString ('');
        Reg.CloseKey;
      end;
    end
    else s := 'Fichier ' + UpperCase(Ext);
  finally
    Reg.Free;
  end;
  result := s;
end;

function FileIcon(FileName : TFileName): TIcon;
var
  Reg   : TRegistry;
  s, Ext:string;
  FileN : TFileName;
  Indx  : Integer;
begin
  Ext := ExtractFileExt(FileName); Ext := Copy(Ext, 2, length(Ext) - 1);
  s   := '';
  reg := tRegistry.Create;
  try
    Reg.RootKey := HKEY_CLASSES_ROOT;
    if Reg.OpenKey('.' + ext + cRegDefaultIcon, false) then
    begin
      s := reg.ReadString('');
      reg.CloseKey;
    end else
    begin
      if Reg.OpenKey('.' + ext, false) then
      begin
        s := reg.ReadString('');
        reg.CloseKey;
        if s <> '' then
        begin
          if Reg.OpenKey(s + cRegDefaultIcon, false)
            then s := reg.ReadString('');
          reg.CloseKey;
        end;
      end;
    end;
    if Pos(',', s) <> 0 then
    begin
      FileN := Copy(s, 1, Pos(',', s) - 1);
      Indx  := StrToInt(Copy(s, pos(',', s) + 1, length(s)));
    end else
    begin
      FileN := s;
      Indx  := 0;
    end;
    if FileN <> ''
      then result := ShellIconExtract(FileN, Indx)
      else result := nil;
  finally
    Reg.Free;
  end;
end;

function NormalDir (Path : string) : string;
begin
  Result := Path;
  if (Result <> '') and not (AnsiLastChar(Result)^ in ['\']) then
  begin
    if (Length(Result) = 1) and (UpCase(Result[1]) in ['A'..'Z']) then
      Result := Result + ':\'
    else Result := Result + '\';
  end;
  Path := Result;
end;

function UnixPathToDosPath(Path: String): string;
var I : integer;
begin
  result := '';
  for i := 1 to length(Path) do
    if Path[i] = '/'
      then result := result + '\'
      else result := result + Path[i];
end;

function DosPathToUnixPath(Path: String): string;
var I : integer;
begin
  result := '';
  for i := 1 to length(Path) do
    if Path[i] = '\'
      then result := result + '/'
      else result := result + Path[i];
end;

function DirExists(const Name: string): Boolean;
var
  Code: Integer;
begin
  Code := GetFileAttributes(PChar(Name));
  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
end;

function GenerateFileName(const Dest: string = '';
                          const Base: string = 'TMP'): string;
var
  PathDest, Prefix, FileName : PChar;
begin
  if Dest = ''
    then PathDest := PChar(GetTempDir)
    else PathDest := PChar(Dest);
  Prefix := PChar(Base);
  GetMem(FileName, MAX_PATH);
  try
    GetTempFileName(PathDest, Prefix, 0, FileName);
    Result := StrPas(FileName);
  finally
    FreeMem(FileName);
  end;
end;

function FileIsUse(FileName: TFileName) : boolean;
var F: File;
begin
  AssignFile(F, FileName);
  {$I-}
  FileMode := 2;
  Reset(F);
  {$I+}
  if IOResult <> 0
    then result := true
    else begin
           result := false;
           CloseFile(F);
         end;
end;

function FileIcon    (FileName : TFileName; getSmallIcon,
                                            getOpenIcon: Boolean)    : HIcon; overload;
var shfi: TShFileInfo;
    flags: Integer;
begin
  flags := SHGFI_ICON;
  if getSmallIcon then flags := flags or SHGFI_SMALLICON;
  if getOpenIcon then flags := flags or SHGFI_OPENICON;
  try
    FillChar(shfi, SizeOf(TShFileInfo), 0);
    ShGetFileInfo(PChar(Filename), 0, shfi, SizeOf(TShFileInfo), flags);
    result := shfi.hIcon;
  except
    result := 0;
  end;
end;

end.
