(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit GradientBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TBarBorderStyle      = (bbsNone, bbsLowered, bbsRaised);
  TGradientBarProgress = class(TGraphicControl)
  private
    { Dclarations prives }
    FColorList : TStrings;
    FColor     : TColor;
    FStyle     : TBarBorderStyle;
    FMin, FMax,
    FProgress  : Integer;
    FBitmap    : TBitmap;
    FCaption   : string;
  protected
    { Dclarations protges }
    procedure setCaption   (Value : string);
    procedure SetColorList (Value : TStrings);
    procedure SetColor     (Value : TColor);
    procedure SetStyle     (Value : TBarBorderStyle);
    procedure SetProgressValues (Index : Integer; Value : Integer);
    procedure PaintBitmap(Sender: TObject);

    procedure Paint; override;
  public
    { Dclarations publiques }
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;

    property Progress  : Integer index 3 read FProgress write SetProgressValues default 0;
  published
    { Dclarations publies }
    property Caption   : string   read FCaption   write setCaption;
    property ColorList : TStrings read FColorList write SetColorList;
    property Color     : TColor   read FColor     write SetColor default clSilver;
    property Position  : Integer index 3 read FProgress write SetProgressValues default 0;
    property Max       : Integer index 2 read FMax      write SetProgressValues default 100;
    property Min       : Integer index 1 read FMin      write SetProgressValues default 0;
    property Style     : TBarBorderStyle read FStyle    write SetStyle default bbsLowered;

    property Height default 12;
    property Width default 150;
    property DragCursor;
    property DragMode;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

uses PaintUtils;

{---------------- Constructeur / Destructeur ----------------------}

constructor TGradientBarProgress.Create (AOwner : TComponent);
begin
  inherited Create (AOwner);
  FColorList := TStringList.Create;
  FBitmap    := TBitmap.Create;
  Color      := clSilver;
  Style      := bbsLowered;
  OnResize   := PaintBitmap;
  Max        := 100;
  Height     := 12;
  Width      := 150;
end;

destructor TGradientBarProgress.Destroy;
begin
  FBitmap.Free;
  FColorList.Free;
  inherited Destroy;
end;

{---------------------- Procedure Set des proprites --------------}

procedure TGradientBarProgress.SetStyle (Value : TBarBorderStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    Refresh;
  end;
end;

procedure TGradientBarProgress.SetColor (Value : TColor);
begin
  if Value <> FColor then
  begin
    FColor := Value;
    Refresh;
  end;
end;

procedure TGradientBarProgress.setCaption (Value : string);
begin
  if Value <> FCaption then
  begin
    FCaption := Value;
    if Assigned (Parent) then
    Paint;
  end;
end;

procedure TGradientBarProgress.SetProgressValues (Index : Integer; Value : Integer);
begin
  Case Index of
    1 : if Value < FMax                        then FMin      := Value;
    2 : if Value > FMin                        then FMax      := Value;
    3 : if (Value >= FMin) and (Value <= FMax) then FProgress := Value;
    else Exit;
  end;
  if Assigned (Parent) then
  Paint;
end;

procedure TGradientBarProgress.SetColorList (Value : TStrings);
begin
  if FColorList <> Value then
  begin
    FColorList.Assign (Value);
    PaintBitmap(Self);
    Refresh;
  end;
end;

{------------------------ Dessine le contrle -------------------}

procedure TGradientBarProgress.PaintBitmap(Sender: TObject);
var I, WidthOfGradient : Integer;
    R                  : TRect;
begin
  if (Height = 0) or (Width = 0) then Exit;
  FBitmap.Height := Height - 2;
  FBitmap.Width  := Width - 2;
  if ColorList.Count < 2 then
  begin
    ColorList.Add ('clBlack');
    if ColorList.Count = 1
      then ColorList.Add ('clBlue');
  end;
  WidthOfGradient := (FBitmap.Width div (ColorList.Count - 1));
  For I := 0 to ColorList.Count - 2 do
  begin
    R := Rect (I * WidthOfGradient, 0, (I * WidthOfGradient) + WidthOfGradient,
               FBitmap.Height);
    GradientFillRect(FBitmap.Canvas, R, StringToColor (ColorList.Strings [I]),
                     StringToColor (ColorList.Strings [I + 1]), fdLeftToRight, 255);
  end;
end;

procedure TGradientBarProgress.Paint;
var Percent, X, Y : Integer;
    R             : TRect;
begin
  if (FBitmap.Empty) or (FBitmap.Width  <> Width)
                     or (FBitmap.Height <> Height)
    then PaintBitmap(Self);
  with Canvas do
  begin
     if Style <> bbsNone then
     begin
       Brush.Color := Color;
       Pen.Width   := 2;
       if Style = bbsLowered
         then Pen.Color := clGray
         else Pen.Color := clWhite;
       MoveTo (0, Height);
       LineTo (0, 0);
       LineTo (Width, 0);

       if Style = bbsLowered
         then Pen.Color := clWhite
         else Pen.Color := clGray;
       LineTo (Width, Height);
       LineTo (0, Height);
     end;
     Pen.Width   := 0;
     Brush.Color := Color;
     Pen.Color   := Color;
     Rectangle (1, 1, Width - 1, Height - 1);

     {------------- Progression -------------}
     if Max = Min then Exit;
     Percent := (Progress * 100) div (Max - Min);
     R       := Rect (1, 1, (Percent * Width) div 100, Height - 2);
     Canvas.BrushCopy (R, FBitmap, R, clNone);

     {------------- Caption ----------------}
     Canvas.Brush.Style := bsClear;
     X       := (Height div 2) - (Canvas.TextHeight(FCaption) div 2);
     Y       := (Width div 2)  - (Canvas.TextWidth(FCaption) div 2);
     Canvas.TextOut(Y, X, FCaption);
  end;
end;

end.
