(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit VerManag;

interface

type
  tVersionFile  = record
                    Majeure,
                    Mineure,
                    Sous_Version,
                    Construction  : integer;
                  end;

  tVersionsInfo = (viVersion, viComments, viDescription, viCopyrigth,
                   viProducteName, viProducteVers, viCompany, viOther);

function GetRessourceFile (FileName: string; VerInfo : tVersionsInfo;CommandLine: string = '') : string;

function ComposeVersion (Version : string) : tVersionFile;
function DecomposeVersion   (Version : tVersionFile) : string;
function GetVersion : TVersionFile;
function CompareVersion (V1, V2 : TVersionFile) : integer;

function HeadGenerator : string;
function GetWindowsVersion: string;

// Pour le numro de version de Windows
{$IFDEF Win32}
resourcestring
  sWindowsVersion = 'Windows %s %d.%.2d.%.3d %s';
{$ELSE}
const
  sWindowsVersion = 'Windows%s %d.%d';
{$ENDIF}
  CopyrightLn = '%s v%d.%d, Copyright %s';

implementation

uses Windows, {$IFDEF Win32} VersionFileType, {$ELSE} Ver, {$ENDIF} SysUtils;

function GetRessourceFile (FileName: string; VerInfo : tVersionsInfo;CommandLine: string = '') : string;
var Version : tVersionInfo;
begin
  Version := nil;
  try
    Version := TVersionInfo.Create (FileName);
    case VerInfo of
      viVersion      : result := LongVersionToString(Version.FileLongVersion);
      viComments     : result := Version.Comments;
      viDescription  : result := Version.FileDescription;
      viCopyrigth    : result := Version.LegalCopyright;
      viProducteName : result := Version.ProductName;
      viProducteVers : result := Version.ProductVersion;
      viCompany      : result := Version.CompanyName;
      else             result := Version.Values [CommandLine];
    end;
  except
    Result := '';
  end;
  Version.Free;
end;

function ComposeVersion (Version : string) : tVersionFile;
var I, iVersion, PosPoint : integer;
begin
  if Version = '' then Version := '0.0.0.0';
  Result.Majeure := 0; Result.Mineure := 0; Result.Sous_Version := 0; Result.Construction := 0;
  I              := 0;
  while Version <> '' do
  begin
    Inc(I);
    PosPoint := Pos ('.', Version);
    if PosPoint = 0
      then PosPoint := Length (Version) + 1;
    iVersion := StrToInt(Copy(Version, 1, PosPoint - 1));
    Delete (Version, 1, PosPoint);
    case I of
      1 : Result.Majeure      := iVersion;
      2 : Result.Mineure      := iVersion;
      3 : Result.Sous_Version := iVersion;
      4 : Result.Construction := iVersion;
    end;
  end;
end;

function DecomposeVersion (Version : tVersionFile) : string;
begin
  result := IntToStr (Version.Majeure)      + '.' +
            IntToStr (Version.Mineure)      + '.' +
            IntToStr (Version.Sous_Version) + '.' +
            IntToStr (Version.Construction);
end;

function GetVersion : TVersionFile;
begin
  result := ComposeVersion (GetRessourceFile (ParamStr(0), viVersion));
end;

function CompareVersion (V1, V2 : TVersionFile) : integer;
var I, iVersionA, iVersionB : integer;
begin
  result := 0;
  iVersionA := 0;
  iVersionB := 0;
  for I := 1 to 4 do
  begin
    case I of
      1 : begin
            iVersionA := V1.Majeure;
            iVersionB := V2.Majeure;
          end;
      2 : begin
            iVersionA := V1.Mineure;
            iVersionB := V2.Mineure;
          end;
      3 : begin
            iVersionA := V1.Sous_Version;
            iVersionB := V2.Sous_Version;
          end;
      4 : begin
            iVersionA := V1.Construction;
            iVersionB := V2.Construction;
          end;
    end;
    if iVersionA < iVersionB
      then result := -1;
    if iVersionA > iVersionB
      then result := 1;
    if result <> 0 then Break;
  end;
end;

function HeadGenerator : string;
var
  Vers : tVersionFile;
begin
  Vers  := GetVersion;
  result := Format (CopyrightLn, [GetRessourceFile (ParamStr(0), viProducteName),
                                  Vers.Majeure, Vers.Mineure,
                                  GetRessourceFile (ParamStr(0), viCopyrigth)]);
end;

function GetWindowsVersion: string;
{$IFDEF WIN32}
var
  Ver: TOsVersionInfo;
  Platform: string[4];
begin
  Ver.dwOSVersionInfoSize := SizeOf(Ver);
  GetVersionEx(Ver);
  with Ver do
  begin
    case dwPlatformId of
      VER_PLATFORM_WIN32s        : Platform := '32s';
      VER_PLATFORM_WIN32_WINDOWS : begin
                                     dwBuildNumber := dwBuildNumber and $0000FFFF;
                                     if (dwMajorVersion = 4)
                                       then if dwMinorVersion >= 10
                                              then if dwMinorVersion >= 90
                                                     then Platform := 'Me'
                                                     else Platform := '98'
                                              else Platform := '95'
                                       else Platform := '';
                                   end;
      VER_PLATFORM_WIN32_NT      : begin
                                     dwBuildNumber := dwBuildNumber and $0000FFFF;
                                     if (dwMajorVersion >= 5)
                                       then if dwMinorVersion >= 01
                                              then Platform := 'XP'
                                              else Platform := '2000'
                                       else Platform := 'NT';
                                   end;
    end;
    Result := Trim(Format(sWindowsVersion, [Platform, dwMajorVersion,
      dwMinorVersion, dwBuildNumber, szCSDVersion]));
  end;
end;
{$ELSE}
const
  sNT: array[Boolean] of string[3] = ('', ' NT');
var
  Ver: Longint;
begin
  Ver := GetVersion;
  Result := Format(sWindowsVersion, [sNT[not Boolean(HiByte(LoWord(Ver)))],
    LoByte(LoWord(Ver)), HiByte(LoWord(Ver))]);
end;
{$ENDIF WIN32}

end.
