(**************************************************************************************)
(*  'tils Lib - Librairie de composants divers                                        *)
(*  Copyright (C) 2003  U. Van Den Hekke                                              *)
(*                                                                                    *)
(*  This library is free software; you can redistribute it and/or                     *)
(*  modify it under the terms of the GNU Lesser General Public                        *)
(*  License as published by the Free Software Foundation; either                      *)
(*  version 2.1 of the License, or (at your option) any later version.                *)
(*                                                                                    *)
(*  This library is distributed in the hope that it will be useful,                   *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of                    *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                 *)
(*  Lesser General Public License for more details.                                   *)
(*                                                                                    *)
(*  You should have received a copy of the GNU Lesser General Public                  *)
(*  License along with this library; if not, write to the Free Software               *)
(*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA         *)
(**************************************************************************************)

unit WinInfos;

interface

(* Dossier *)

function GetWindowsDir : string;
function GetSystemDir  : string;
function GetTempDir    : string;
{$IFDEF WIN32}
function GetCommonDir  : string;
function GetMyPictures : string;

(* Fonction Administrateur *)

function IsAdminLoggedOn: Boolean;
{$ENDIF}

implementation

uses Windows, {$IFDEF WIN32}Registry, {$ENDIF} SysUtils;

resourcestring
  // Clef et valeur de l'inscription du rpertoire en partage
  PathCommon  = 'Software\Microsoft\Windows\CurrentVersion\';
    RegCommon = 'CommonFilesDir';
  PathImages  = 'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders\';
    RegImages = 'My Pictures';

function GetWindowsDir: string;
{$IFDEF WIN32}
var
  Buffer: array[0..1023] of Char;
begin
  SetString(Result, Buffer, GetWindowsDirectory(Buffer, SizeOf(Buffer)));
{$ELSE}
begin
  Result[0] := Char(GetWindowsDirectory(@Result[1], 254));
{$ENDIF}
end;

function GetSystemDir: string;
{$IFDEF WIN32}
var
  Buffer: array[0..1023] of Char;
begin
  SetString(Result, Buffer, GetSystemDirectory(Buffer, SizeOf(Buffer)));
{$ELSE}
begin
  Result[0] := Char(GetSystemDirectory(@Result[1], 254));
{$ENDIF}
end;

function GetTempDir: string;
{$IFDEF WIN32}
var
  Buffer: array[0..1023] of Char;
begin
  SetString(Result, Buffer, GetTempPath(SizeOf(Buffer), Buffer));
{$ELSE}
var
  Buffer: array[0..255] of Char;
begin
  GetTempFileName(GetTempDrive(#0), '$', 1, Buffer);
  Result := ExtractFilePath(StrPas(Buffer));
{$ENDIF}
end;

{$IFDEF WIN32}
function GetCommonDir : string;
var RegFile : TRegistry;
begin
  RegFile         := TRegistry.Create;
  RegFile.RootKey := HKEY_LOCAL_MACHINE;
  RegFile.OpenKey (PathCommon, False);
  result := RegFile.ReadString (RegCommon);
  RegFile.CloseKey;
  RegFile.Free;
end;

function GetMyPictures(): string;
var RegFile : TRegistry;
begin
  RegFile         := TRegistry.Create;
  RegFile.RootKey := HKEY_CURRENT_USER;
  RegFile.OpenKey (PathImages, false);
  result := RegFile.ReadString (RegImages);
  RegFile.CloseKey;
  RegFile.Free;
end;

type
  SC_HANDLE = THandle;

function OpenSCManager(lpMachineName, lpDatabaseName: PChar;
                       dwDesiredAccess: DWORD): SC_HANDLE; stdcall; external 'advapi32.dll' name 'OpenSCManagerA';
function CloseServiceHandle(hSCObject: SC_HANDLE): BOOL; stdcall;   external 'advapi32.dll' name 'CloseServiceHandle';

{ Returns True if an administrator is logged onto the system. Always returns
  True on Windows 95/98. }
function IsAdminLoggedOn: Boolean;
var hSC: SC_HANDLE;
begin
  if Win32Platform <> VER_PLATFORM_WIN32_NT then Result := True else
  begin
    { Try an admin privileged API }
    hSC := OpenSCManager(nil, nil, GENERIC_READ or GENERIC_WRITE or GENERIC_EXECUTE);
    Result := hSC <> 0;
    if Result then CloseServiceHandle(hSC);
  end;
end;
{$ENDIF}

end.
